#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266HTTPClient.h>
#include <WiFiClient.h>

#ifndef APSSID
#define APSSID "ESPap"
#define APPSK "thereisnospoon"
#endif

#define BUTTON D2

/* Set these to your desired credentials. */
const char *APssid = APSSID;
const char *APpassword = APPSK;

const char *ssid = "QueirozX";
const char *password = "queirozlindo";

bool is_discovering_tuya_ip = true;


ESP8266WebServer web_server(80);
WiFiClient wifiClient;

String HOST_ADDRESS = "192.168.0.47";

String make_toggle_request()
{
  HTTPClient http;
  String ADCData, station, getData, request;

  request = "http://" + HOST_ADDRESS + "/cm?cmnd=Power24%20Toggle";
  Serial.println("Sent request to:" + request);
  http.begin(wifiClient, request);

  auto httpCode = http.GET();
  String response = http.getString();
  http.end();

  return response;
}

void handleRoot()
{
  String response = make_toggle_request();
  web_server.send(200, "text/html", response);
}

void handleSetHost()
{
  String host = web_server.arg("host");
  Serial.println("Setting host to: " + host);
  HOST_ADDRESS = host;
}


void try_to_connect_to_wifi_network_if_not_connected()
{
  WiFi.begin(ssid, password);
  while(WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.println("");
}

String  return_first_client_ip()
{
  unsigned char softap_stations_cnt;
  struct station_info *stat_info;
  uint32 uintaddress;

  softap_stations_cnt = wifi_softap_get_station_num();
  stat_info = wifi_softap_get_station_info();
  String string_ip;
  
  if (stat_info != NULL) {
    auto IPaddress = &stat_info->ip;
    uintaddress = IPaddress->addr;
    IPAddress myIp(uintaddress);
    string_ip = myIp.toString();
  }
  else
  {
    return String("👎");
  }
  return string_ip;
}

void setup()
{
  pinMode(BUTTON, INPUT_PULLUP);
  delay(1000);
  Serial.begin(9600);
  Serial.println("Init");
  Serial.print("Configuring access point...");
  /* You can remove the password parameter if you want the AP to be open. */

  WiFi.mode(WIFI_AP_STA);
  WiFi.softAP(APssid, APpassword);
  
  Serial.print("Server ip: ");
  Serial.println(WiFi.softAPIP());

  Serial.print("AP IP address: ");
  web_server.on("/", handleRoot);
  web_server.on("/sethost", handleSetHost);
  web_server.begin();
  Serial.println("HTTP server started");


  while (is_discovering_tuya_ip)
  {
    Serial.println('Collecting ip');
    String client_ip = return_first_client_ip();
    if (client_ip != String("👎"))
    {
      is_discovering_tuya_ip = false;
      HOST_ADDRESS = client_ip;
      Serial.println("New host address: " + HOST_ADDRESS);
    }
  }
  Serial.println("Leaving setup");

  try_to_connect_to_wifi_network_if_not_connected();
  Serial.println(WiFi.localIP());
  Serial.println(WiFi.softAPIP());

}


void handle_button_click()
{
  if(digitalRead(BUTTON) == LOW) {
    make_toggle_request();
    delay(500);
  }
}


void loop()
{
  handle_button_click();
  web_server.handleClient();
}
